<?php
namespace Teknisa\Model\Geral\Repositories;
use Teknisa\Model\Traits\TQuery;
use Doctrine\DBAL\Connection;

      
class Cliente extends \Doctrine\ORM\EntityRepository {
    use TQuery;

    public function findAllClientes()
    {
        $query  = <<<SQL
            SELECT DISTINCT
                   C.CDCLIENTE, C.NRINSJURCLIE, C.NMFANTCLIE, C.IDTPIJURCLIE, OC.CDCLIENTE IDPOSSUI, C.NMRAZSOCCLIE,
                   SLA.ID_CONTROLA_SLA, SLA.ID_PARAM_SLA
              FROM CLIENTE C
         LEFT JOIN (SELECT DISTINCT CDCLIENTE, IDTPOPERCLIE
                      FROM OPERCLIE) OC ON OC.CDCLIENTE = C.CDCLIENTE
         LEFT JOIN SLACLIENTE_PARAM SLA ON C.CDCLIENTE = SLA.CDCLIENTE
             WHERE C.IDCLIEATIV <> 'N'
          ORDER BY C.NMFANTCLIE
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $query);
    }
    
    public function findAllVersions()
    {
        $query  = <<<SQL
           SELECT
               NRVERSAODISP,
               CDVERSAODISP,
               CDRELVERSAODISP,
               CDVERSAODISP||'.'||CDRELVERSAODISP AS VERSION 
            FROM VERSAODISP
           ORDER BY
               CDVERSAODISP DESC,
               CDRELVERSAODISP DESC
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $query);
    }
    
    public function findClientesWithLatestVersion()
    {
        $query  = <<<SQL
            SELECT DISTINCT 
                   C.CDCLIENTE, C.NMFANTCLIE
                   ,( ( SELECT
                       *
                   FROM( SELECT 
                   P.CDVERSAODISP||'.'||CDRELVERSAODISP
                   FROM (SELECT 
                             ROWNUM AS ROW_NUM,
                             VDC.NRVERSAODISP,
                             VD.CDVERSAODISP,
                             VD.CDRELVERSAODISP,
                             TO_CHAR(VDC.DTINCVERSAOCLI) AS DATAINCLUSAO,
                             OP.NMOPERADOR
                          FROM
                             VERSAODISPCLI VDC,
                             VERSAODISP VD,
                             CLIENTE CL,
                             OPERADOR OP
                         WHERE
                             VDC.NRVERSAODISP    = VD.NRVERSAODISP AND
                             VDC.CDCLIENTE       = CL.CDCLIENTE AND
                             VDC.CDOPERINCCADCLI = OP.CDOPERADOR AND
                             CL.IDCADVERSAOCLI   = 'S' AND
                             VDC.CDCLIENTE       = C.CDCLIENTE 
                     ORDER BY
                             VD.CDVERSAODISP DESC,
                             VD.CDRELVERSAODISP DESC  )P WHERE ROWNUM = 1  ))
                             ) AS LAST_VERSION
              FROM CLIENTE C
             WHERE C.IDCLIEATIV <> 'N' AND C.IDCADVERSAOCLI   = 'S'
          ORDER BY C.NMFANTCLIE
SQL;

        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $query);
    }
    
    public function findVersionByCliente ($cdcliente)
    {
        $query = <<<SQL
        SELECT
             VDC.NRVERSAODISP,
             VD.CDVERSAODISP,
             VD.CDRELVERSAODISP,
             TO_CHAR(VDC.DTINCVERSAOCLI) AS DATAINCLUSAO,
             OP.NMOPERADOR,
             VD.CDVERSAODISP||'.'||VD.CDRELVERSAODISP AS VERSION 
          FROM
             VERSAODISPCLI VDC,
             VERSAODISP VD,
             CLIENTE CL,
             OPERADOR OP
         WHERE
             VDC.NRVERSAODISP    = VD.NRVERSAODISP AND
             VDC.CDCLIENTE       = CL.CDCLIENTE AND
             VDC.CDOPERINCCADCLI = OP.CDOPERADOR AND
             CL.IDCADVERSAOCLI   = 'S' AND
             VDC.CDCLIENTE       = :P_CODCLIENTE
     ORDER BY
             VD.CDVERSAODISP DESC,
             VD.CDRELVERSAODISP DESC
SQL;
        $params = array(
            'P_CODCLIENTE' => $cdcliente
        );
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $query, $params);
    }
    
    public function deleteAllVersionByCliente ($cdclientesString)
    {
        $query = <<<SQL
        DELETE VERSAODISPCLI
         WHERE CDCLIENTE IN $cdclientesString 
SQL;
        $std = $this->getEntityManager()->getConnection()->prepare($query);
        $std->execute();
    }

    public function findClientesPorOperador($cdOperador)
    {
        $query  = <<<SQL
            SELECT OC.CDOPERADOR, OC.CDCLIENTE, C.NRINSJURCLIE, C.NMFANTCLIE, OC.NRPRIOPERCLI, C.IDTPIJURCLIE, OC.IDEMAILENCSO,
                   DECODE(OC.IDTPOPERCLIE, 'R', 'Responsável', 'C', 'Cliente') AS IDTPOPERCLIE
              FROM OPERCLIE OC, CLIENTE C
             WHERE OC.CDOPERADOR = :P_CDOPERADOR
               AND OC.CDCLIENTE = C.CDCLIENTE
          ORDER BY C.NMFANTCLIE
SQL;

        $param = array(
            'P_CDOPERADOR' => $cdOperador
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $query, $param);
    }
    
    public function includeVersionsForClients($nrversaodisp, $cdcliente, $cdOperador) {
        $query = <<<SQL
            INSERT INTO VERSAODISPCLI
                   (NRVERSAODISP, CDCLIENTE, CDOPERINCCADCLI, DTINCVERSAOCLI, CDOPERINCLUSAO, DTINCLUSAO)
            VALUES
                   (:P_NRVERSAODISP, :P_CDCLIENTE, :P_CDOPERADOR, SYSDATE, :P_CDOPERADOR, SYSDATE)

SQL;
         $param = array(
            'P_NRVERSAODISP' => $nrversaodisp,
            'P_CDCLIENTE' => $cdcliente,
            'P_CDOPERADOR' => $cdOperador
        );
        $this->getEntityManager()->getConnection()->executeQuery($query, $param);
    }
    
     public function includeVersionsForAllClients($nrversaodisp, $cdcliente, $cdOperador) {
        $query = <<<SQL
            INSERT INTO VERSAODISPCLI
                   (NRVERSAODISP, CDCLIENTE, CDOPERINCCADCLI, DTINCVERSAOCLI, CDOPERINCLUSAO, DTINCLUSAO)
            VALUES
                   (:P_NRVERSAODISP, :P_CDCLIENTE, :P_CDOPERADOR, SYSDATE, :P_CDOPERADOR, SYSDATE)

SQL;
         $param = array(
            'P_NRVERSAODISP' => $nrversaodisp,
            'P_CDCLIENTE' => $cdcliente,
            'P_CDOPERADOR' => $cdOperador
        );
        $this->getEntityManager()->getConnection()->executeQuery($query, $param);
    }
    
    public function deleteVersionsForClients($nrversaodisp, $cdcliente)
    {
        $query = <<<SQL
        DELETE VERSAODISPCLI
         WHERE CDCLIENTE = :P_CDCLIENTE AND NRVERSAODISP IN (:P_NRVERSAODISP)
SQL;
        
        $param = array(
            ':P_NRVERSAODISP' => $nrversaodisp,
            ':P_CDCLIENTE' => $cdcliente
        );
        $paramTypes = array(
            ':P_NRVERSAODISP' => Connection::PARAM_STR_ARRAY
        );
       $this->getEntityManager()->getConnection()->executeQuery($query, $param, $paramTypes);
    }
    
    public function deleteVersionsForAllClients($nrversaodisp, $cdcliente)
    {
        $query = <<<SQL
        DELETE VERSAODISPCLI
         WHERE CDCLIENTE = :P_CDCLIENTE AND NRVERSAODISP IN (:P_NRVERSAODISP)
SQL;
        
        $param = array(
            ':P_NRVERSAODISP' => $nrversaodisp,
            ':P_CDCLIENTE' => $cdcliente
        );
        $paramTypes = array(
            ':P_NRVERSAODISP' => Connection::PARAM_STR_ARRAY
        );
       $this->getEntityManager()->getConnection()->executeQuery($query, $param, $paramTypes);
    }

    public function findRelacionarClientesComOperador($cdoperador){
        $sql = <<<SQL
               SELECT CDCLIENTE, NMFANTCLIE, NMRAZSOCCLIE
                 FROM CLIENTE
                WHERE IDCLIEATIV = 'S'
                  AND CDCLIENTE NOT IN (SELECT OC.CDCLIENTE
                                          FROM OPERCLIE OC, CLIENTE C
                                         WHERE OC.CDOPERADOR = :P_CDOPERADOR
                                           AND OC.CDCLIENTE = C.CDCLIENTE)
             ORDER BY NMFANTCLIE
SQL;
        $param = array(
            'P_CDOPERADOR' => $cdoperador,
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $param);
    }

    public function findParamSla($limit, $offset,$cdcliente){
        $query = <<<SQL
            SELECT CDASSUNTO, CDGRUPFASE, IDPRAZOSLA, QTDIASSLA
              FROM SLACLIENTE
             WHERE CDCLIENTE = :P_CDCLIENTE
SQL;
        if($limit){
            $query  = self::getPaginatedQuery($query, $limit, $offset);
        };
        $param = array(
            'P_CDCLIENTE' => $cdcliente,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$param);
    }

    public static function getPaginatedQuery($query, $limit, $offset) {
        if ($limit > 0) {
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $limit . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $limit;
            }
        }
        return $query;
    }
}
